import { InjectionToken, Directive, ViewContainerRef, TemplateRef, Optional, Inject, Input, NgModule, Injectable } from '@angular/core';
import { isFunction } from 'lodash';
import { Logger } from '@nyy/logger';
import { CommonModule } from '@angular/common';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import * as Url_ from 'url';
import * as qs_ from 'qs';
import { HttpResponse, HttpClient } from '@angular/common/http';
import { of } from 'rxjs';

const logger = new Logger('CORE:PRIVILEGE');
const privilegeAppInjector = { injector: null };
const privilegeInjectorRegister = (injector) => {
    Object.defineProperty(privilegeAppInjector, 'injector', {
        value: injector,
        enumerable: true,
        writable: false,
        configurable: false,
    });
};
const PrivilegeRequire = (privileges) => {
    return (target, propertyKey, descriptor) => {
        if (isFunction(descriptor.value)) {
            const originalMethod = descriptor.value;
            descriptor.value = function (...args) {
                const injector = privilegeAppInjector.injector;
                logger.debug('privilegeAppInjector: %o', injector);
                const service = injector.get(PrivilegeServiceToken, null);
                logger.debug('privilege service: %o', service);
                if (service && service.hasPrivilege(privileges)) {
                    // skip privilege
                    return originalMethod.apply(this, args);
                }
                else if (!service) {
                    return originalMethod.apply(this, args);
                }
            };
        }
    };
};
const PrivilegeServiceToken = new InjectionToken('privilege_service_token');

const logger$1 = new Logger('CORE:PermissionRequire');
class PrivilegeRequireDirective {
    constructor(viewContainerRef, templateRef, privilegeService) {
        this.viewContainerRef = viewContainerRef;
        this.templateRef = templateRef;
        this.privilegeService = privilegeService;
    }
    set privilegeRequire(v) {
        if (!this.privilegeService) {
            logger$1.debug('privilege service is not defined; should you register the privilege provider?');
            this.viewContainerRef.createEmbeddedView(this.templateRef);
        }
        else if (this.privilegeService.hasPrivilege(v)) {
            this.viewContainerRef.createEmbeddedView(this.templateRef);
        }
        else {
            this.viewContainerRef.clear();
        }
    }
}
PrivilegeRequireDirective.decorators = [
    { type: Directive, args: [{
                selector: '[privilegeRequire]',
            },] }
];
PrivilegeRequireDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: TemplateRef },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [PrivilegeServiceToken,] }] }
];
PrivilegeRequireDirective.propDecorators = {
    privilegeRequire: [{ type: Input }]
};

class PrivilegeModule {
}
PrivilegeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [PrivilegeRequireDirective],
                imports: [
                    CommonModule,
                ],
                exports: [PrivilegeRequireDirective],
            },] }
];

const logger$2 = new Logger('CORE:I18N');
const TRANSLATE_CONFIG = new InjectionToken('ngx-translate-config');
const NgxTranslateHttpLoaderFactory = (http, config) => {
    logger$2.debug('[NgxTranslateHttpLoaderFactory][args]: %o', config);
    if (config) {
        logger$2.debug('[TRANSLATE_CONFIG]: %o', config);
        return new TranslateHttpLoader(http, config.path, config.suffix);
    }
    return new TranslateHttpLoader(http, '/assets/i18n/', '.json');
};

const MOCK_CONFIG = new InjectionToken('mock_config');

const Url = Url_;
const qs = qs_;
const logger$3 = new Logger('CORE:Mock');
class MockInterceptor {
    constructor(http, config = {}) {
        this.http = http;
        this.config = config;
    }
    intercept(request, next) {
        const mocks = this.config.modules;
        if (mocks && Object.keys(mocks).length) {
            const url = Url.parse(request.url);
            const method = request.method;
            const pathname = url.pathname;
            const mockPathWithMethod = `${method} ${pathname}`;
            const mockDataGenerator = mocks[mockPathWithMethod];
            if (mockDataGenerator) {
                const query = qs.parse(url.query);
                const resBody = mockDataGenerator(query, request.params, request.body);
                logger$3.debug('[Mock INFO]' +
                    '\n[Mock Request Url] => %s::%s ' +
                    '\n[Mock Request Body] => %o' +
                    '\n[Mock Response Body] => %o', request.method, request.url, request.body, resBody);
                return of(new HttpResponse({
                    status: 200,
                    body: resBody,
                }));
            }
        }
        return next.handle(request);
    }
}
MockInterceptor.decorators = [
    { type: Injectable }
];
MockInterceptor.ctorParameters = () => [
    { type: HttpClient },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MOCK_CONFIG,] }] }
];

/*
 * Public API Surface of core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MOCK_CONFIG, MockInterceptor, NgxTranslateHttpLoaderFactory, PrivilegeModule, PrivilegeRequire, PrivilegeRequireDirective, PrivilegeServiceToken, TRANSLATE_CONFIG, privilegeInjectorRegister };
//# sourceMappingURL=nyy-core.js.map
